package springbatchdemo;

import java.util.Map;
import org.junit.*;
import org.junit.runner.RunWith;
import org.springframework.batch.core.*;
import org.springframework.batch.core.explore.JobExplorer;
import org.springframework.batch.core.launch.JobOperator;
import org.springframework.batch.item.ExecutionContext;
import org.springframework.beans.factory.annotation.*;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@RunWith( SpringJUnit4ClassRunner.class )
@ContextConfiguration( classes = MeineApplication.class )
public class ConcurrentJobTest
{
   @Autowired private JobExplorer jobExplorer;
   @Autowired @Qualifier( "asyncJobOperator" ) private JobOperator asyncJobOperator;

   /** Test der Multi-threading-Faehigkeit */
   @Test
   public void testConcurrentJob() throws Exception
   {
      // Per asynchronem JobOperator werden zwei parallel laufende JobExecutions gestartet (mit verschiedenen JobParametern):
      Long executionId1 = asyncJobOperator.start( ConcurrentJobConfiguration.JOB_NAME, "MeinParm=a" );
      Long executionId2 = asyncJobOperator.start( ConcurrentJobConfiguration.JOB_NAME, "MeinParm=b" );

      JobExecution je1 = warteBisStatus( executionId1, BatchStatus.COMPLETED, 2 );
      JobExecution je2 = warteBisStatus( executionId2, BatchStatus.COMPLETED, 2 );

      printStepExecutions( je1 );
      System.out.println( "" );
      printStepExecutions( je2 );

      // Da pro JobExecution der "Arbeits-Step" jeweils dreimal durchlaufen wird, wird der JobParameter dreimal gespeichert:
      Assert.assertEquals( "aaa", je1.getExecutionContext().get( ConcurrentJobConfiguration.MEIN_MSG_KEY ) );
      Assert.assertEquals( "bbb", je2.getExecutionContext().get( ConcurrentJobConfiguration.MEIN_MSG_KEY ) );
   }

   /** Warte bis die JobExecution den gewuenschten BatchStatus erreicht (oder der Timeout ueberschritten ist) */
   private JobExecution warteBisStatus( Long executionId, BatchStatus stat, int maxWartezeitSek )
   {
      for( int i = 0; i < 5 * maxWartezeitSek; i++ ) {
         JobExecution je = jobExplorer.getJobExecution( executionId );
         if( stat == null || je == null || stat.equals( je.getStatus() ) ) {
            return je;
         }
         try { Thread.sleep( 200 ); } catch( InterruptedException e ) {}
      }
      return jobExplorer.getJobExecution( executionId );
   }

   /** Zeige die StepExecutions, die JobExecution und den Job-ExecutionContext */
   private static void printStepExecutions( JobExecution je )
   {
      for( StepExecution se : je.getStepExecutions() ) {
         String s = se.getExitStatus().getExitDescription();
         if( s != null && s.length() > 0 ) { s = ", ExitDescription = " + s; }
         System.out.println( "StepExecution " + se.getId() + ": CommitCount = " + se.getCommitCount() +
               ", Status = " + se.getStatus() + ", ExitStatus = " + se.getExitStatus().getExitCode() +
               ", StepName = " + se.getStepName() + s );
      }
      String s = je.getExitStatus().getExitDescription();
      if( s != null && s.length() > 0 ) { s = ", ExitDescription = " + s; }
      System.out.println( "JobExecution  " + je.getId() + ": JobId = " + je.getJobId() +
            ",       Status = " + je.getStatus() + ", ExitStatus = " + je.getExitStatus().getExitCode() + s );
      ExecutionContext ec = je.getExecutionContext();
      if( ec != null && !ec.isEmpty() ) {
         for( Map.Entry<String, Object> e : ec.entrySet() ) {
            System.out.println( "Job-ExecutionContext: " + e.getKey() + ": " + e.getValue() );
         }
      }
   }
}
