package springbatchdemo;

import org.springframework.batch.core.*;
import org.springframework.batch.core.configuration.annotation.*;
import org.springframework.batch.core.job.builder.FlowBuilder;
import org.springframework.batch.core.job.flow.Flow;
import org.springframework.batch.core.launch.support.RunIdIncrementer;
import org.springframework.batch.core.scope.context.ChunkContext;
import org.springframework.batch.core.step.tasklet.Tasklet;
import org.springframework.batch.repeat.RepeatStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.*;
import org.springframework.core.task.SimpleAsyncTaskExecutor;

@Configuration
@EnableBatchProcessing
public class SplitFlowJobConfiguration
{
   @Autowired private JobBuilderFactory  jobBuilderFactory;
   @Autowired private StepBuilderFactory stepBuilderFactory;

   /** Tasklet: Zeigt Text mehrmals im Kommandozeilenfenster */
   public class PrintTextMehrmalsTasklet implements Tasklet
   {
      final String text;
      final int    anzahl;

      public PrintTextMehrmalsTasklet( String text, int anzahl ) {
         this.text   = text;
         this.anzahl = anzahl;
      }

      @Override
      public RepeatStatus execute( StepContribution sc, ChunkContext cc ) throws Exception {
         for( int i = 0; i < anzahl; i++ ) {
            System.out.print( text );
            Thread.sleep( 200 );
         }
         System.out.print( " " );
         return RepeatStatus.FINISHED;
      }
   }

   @Bean
   public Step step1() {
      return stepBuilderFactory.get( "step1" ).tasklet( new PrintTextMehrmalsTasklet( "1 ", 5 ) ).build();
   }

   @Bean
   public Step step2() {
      return stepBuilderFactory.get( "step2" ).tasklet( new PrintTextMehrmalsTasklet( "2 ", 5 ) ).build();
   }

   @Bean
   public Step step3() {
      return stepBuilderFactory.get( "step3" ).tasklet( new PrintTextMehrmalsTasklet( "3 ", 4 ) ).build();
   }

   @Bean
   public Step step4() {
      return stepBuilderFactory.get( "step4" ).tasklet( new PrintTextMehrmalsTasklet( "4 ", 6 ) ).build();
   }

   @Bean
   public Step step5() {
      return stepBuilderFactory.get( "step5" ).tasklet( new PrintTextMehrmalsTasklet( "5 ", 8 ) ).build();
   }

   @Bean
   public Step step6() {
      return stepBuilderFactory.get( "step6" ).tasklet( new PrintTextMehrmalsTasklet( "6 ", 5 ) ).build();
   }

   @Bean
   public Step step7() {
      return stepBuilderFactory.get( "step7" ).tasklet( new PrintTextMehrmalsTasklet( "7 ", 5 ) ).build();
   }

   /** Job: fuehrt die drei Steps step3, step4 und step5 im "Split Flow" parallel aus */
   @Bean
   public Job meinSplitFlowJob()
   {
      Flow flow3 = new FlowBuilder<Flow>( "flow3" ).from( step3() ).end();
      Flow flow4 = new FlowBuilder<Flow>( "flow4" ).from( step4() ).end();
      Flow flow5 = new FlowBuilder<Flow>( "flow5" ).from( step5() ).end();

      Flow splitFlow345 = new FlowBuilder<Flow>( "splitFlow345" )
              .start( flow3 )
              .split( new SimpleAsyncTaskExecutor() )
              .add(   flow4, flow5 )
              .build();

      return jobBuilderFactory.get( "meinSplitFlowJob" ).incrementer( new RunIdIncrementer() )
              .flow( step1() )
              .next( step2() )
              .next( splitFlow345 )
              .next( step6() )
              .next( step7() )
              .end().build();
   }
}
