package springbatchdemo;

import org.springframework.batch.core.configuration.JobRegistry;
import org.springframework.batch.core.configuration.support.JobRegistryBeanPostProcessor;
import org.springframework.batch.core.explore.JobExplorer;
import org.springframework.batch.core.launch.*;
import org.springframework.batch.core.launch.support.*;
import org.springframework.batch.core.repository.JobRepository;
import org.springframework.beans.factory.annotation.*;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.task.SimpleAsyncTaskExecutor;

@Configuration
public class AsyncJobOperatorConfiguration
{
   @Autowired private JobExplorer   jobExplorer;
   @Autowired private JobRegistry   jobRegistry;
   @Autowired private JobRepository jobRepository;
   @Autowired @Qualifier("asyncJobLauncher") private JobLauncher asyncJobLauncher;

   /** JobRegistry:
    *  Falls in Ihrer Laufzeitumgebung noch keine JobRegistry registriert wurde, wird diese Bean benoetigt.
    *  Falls eine JobRegistry bereits registriert ist, muss diese Bean entfallen. */
   @Bean
   public JobRegistryBeanPostProcessor jobRegistryBeanPostProcessor( JobRegistry jobReg )
   {
      JobRegistryBeanPostProcessor jobRegistryBeanPostProcessor = new JobRegistryBeanPostProcessor();
      jobRegistryBeanPostProcessor.setJobRegistry( jobReg );
      return jobRegistryBeanPostProcessor;
   }

   /** Asynchroner JobLauncher */
   @Bean
   public JobLauncher asyncJobLauncher()
   {
      SimpleJobLauncher jobLauncher = new SimpleJobLauncher();
      jobLauncher.setJobRepository( jobRepository );
      jobLauncher.setTaskExecutor( new SimpleAsyncTaskExecutor() );
      return jobLauncher;
   }

   /** Asynchroner JobOperator */
   @Bean
   public JobOperator asyncJobOperator()
   {
      SimpleJobOperator jobOperator = new SimpleJobOperator();
      jobOperator.setJobExplorer(   jobExplorer );
      jobOperator.setJobRegistry(   jobRegistry );
      jobOperator.setJobRepository( jobRepository );
      jobOperator.setJobLauncher(   asyncJobLauncher );
      return jobOperator;
   }
}
