package springbatchdemo;

import org.springframework.batch.core.*;
import org.springframework.batch.core.configuration.annotation.*;
import org.springframework.batch.core.launch.support.RunIdIncrementer;
import org.springframework.batch.core.scope.context.ChunkContext;
import org.springframework.batch.core.step.tasklet.Tasklet;
import org.springframework.batch.repeat.RepeatStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.*;

@Configuration
@EnableBatchProcessing
public class TaskletJobConfiguration
{
   public static final String ANZAHLSTEPS_KEY = "AnzahlSteps";
   @Autowired private JobBuilderFactory  jobBuilderFactory;
   @Autowired private StepBuilderFactory stepBuilderFactory;

   @Bean
   public Step meinLeerzeilenStep()
   {
      return stepBuilderFactory.get( "meinLeerzeilenStep" ).tasklet( new Tasklet() {
         @Override
         public RepeatStatus execute( StepContribution sc, ChunkContext cc )
         {
            System.out.println( "" );
            return RepeatStatus.FINISHED;
         }
      } ).build();
   }

   @Bean
   public Step meinArbeitsStep()
   {
      return stepBuilderFactory.get( "meinArbeitsStep" ).tasklet( new Tasklet() {
         @Override
         public RepeatStatus execute( StepContribution sc, ChunkContext cc )
         {
            String anz = cc.getStepContext().getStepExecution().getJobParameters().getString( ANZAHLSTEPS_KEY );
            int n = ( anz != null ) ? Integer.parseInt( anz ) : 4;
            int i = cc.getStepContext().getStepExecution().getCommitCount();
            System.out.println( "Hallo Spring Batch mit Tasklet, Tasklet-Execution " + i );
            return ( i < n - 1 ) ? RepeatStatus.CONTINUABLE : RepeatStatus.FINISHED;
         }
      } ).build();
   }

   @Bean
   public Job meinTaskletJob() throws Exception
   {
      return jobBuilderFactory.get( "meinTaskletJob" ).incrementer( new RunIdIncrementer() )
              .start( meinLeerzeilenStep() )
              .next(  meinArbeitsStep() )
              .next(  meinLeerzeilenStep() )
              .build();
   }
}
